import sys, getopt



def generate_problem_grace(index):
    filename = 'lift_controller_'+str(index)+'.smv'
    out = open(filename, 'w')
    
    # Preamble
    print('-- Lift Controller with n='+str(index)+' floors.', file=out)
    print('MODULE main', file=out)

    # EVAR
    print('EVAR', file=out)
    print('\t-- buttons', file=out)
    for i in range(0,index):
        print('\tb'+str(i)+' : boolean;', file=out)
    
    # VAR
    print('VAR', file=out)
    print('\t-- floors', file=out)
    for i in range(0,index):
        print('\tf'+str(i)+' : boolean;', file=out)

    # GR1SPEC
    print('GR1SPEC', file=out)
    print('-- ENVIRONMENT', file=out)

    # ENVINIT
    print('\tENVINIT', file=out)
    print('\t\t-- initially, on all floors there are no requests', file=out)
    print('\t\t', end='', file=out)
    for i in range(0,index):
        end_str = ' ' if ( i==index-1 ) else ' & ' 
        print('!b'+str(i)+end_str, end='', file=out)
    print('', file=out)

    # ENVTRANS
    print('\tENVTRANS', file=out)

    # once a request has been fulfilled, it is removed 
    print('\t\t-- once a request has been fulfilled, it is removed', file=out)
    for i in range(0,index):
        print('\t\t(b'+str(i)+' & f'+str(i)+' -> X !b'+str(i)+') &', file=out)
    # once the lift has been called on some floor, the request
    # cannot be withdrawn
    print('\t\t-- once the lift has been called on some floor, the request', file=out);
    print('\t\t-- cannot be withdrawn', file=out);
    for i in range(0,index):
        end_str = ' ' if (i == index-1) else ' & '
        print('\t\t(b'+str(i)+' & !f'+str(i)+' -> X b'+str(i)+')'+end_str, file=out)
    
    # ENVFAIR
    print('\tENVFAIR', file=out)
    print('\t\tTRUE', file=out)
    print('', file=out)


    # CONTROLLER
    print('-- CONTROLLER', file=out)

    # SYSINIT
    print('\tSYSINIT', file=out)
    print('\t\t-- initially the lift is on the first floor', file=out)
    print('\t\t f0 & ', end='', file=out)
    for i in range(1,index):
        end_str = ' ' if (i == index-1) else ' & '
        print('!f'+str(i)+end_str, end='', file=out)
    print('', file=out)

    # SYSTRANS
    print('\tSYSTRANS', file=out)
    print('\t\t-- the lift should *not* move *up* unless some button is pressed', file=out)
    up_formula = '('
    for i in range(0,index-1):
        end_str = ' ' if (i == index-2) else ' | '
        up_formula += '(f'+str(i)+' & X f'+str(i+1)+')'+end_str
    up_formula += ')'
    sb_formula = '('
    for i in range(0,index):
        end_str = ' ' if (i == index-1) else ' | ' 
        sb_formula += 'b'+str(i)+end_str
    sb_formula += ')'
    print('\t\t'+up_formula+' -> '+sb_formula+' & ', file=out)

    # mutual exclusion
    print('\t\t-- mutual exclusion', file=out)
    for i in range(0, index):
        for j in range(0,i):
            print('\t\t!(f'+str(i)+' & f'+str(j)+') & ', file=out)

    # the lift can move at most one floor at a time (move up, move
    # down, or not move)
    print('\t\t-- the lift can move at most one floor at a time (move up, move',file=out)
    print('\t\t-- down, or not move)', file=out)
    for i in range(0,index):
        if i == 0:
            print('\t\t(f'+str(i)+' -> X(f'+str(i)+' | f'+str(i+1)+')) & ', file=out)
        elif i == index-1:
            print('\t\t(f'+str(i)+' -> X(f'+str(i-1)+' | f'+str(i)+'))', file=out)
        else:
            print('\t\t(f'+str(i)+' -> X(f'+str(i)+' | f'+str(i-1)+' | f'+str(i+1)+')) & ', file=out)

    # SYSFAIR
    for i in range(0,index):
        print('\tSYSFAIR', file=out)
        print('\t\t-- all the requests are eventually satisfied', file=out)
        print('\t\t(b'+str(i)+' -> f'+str(i)+')', file=out)

    # SYSFAIR
    # either some button is pressed infinitely many times, or
    # the lift parks at floor f0 infinitely many times
    print('\tSYSFAIR', file=out)
    print('\t\t-- either some button is pressed infinitely many times, or', file=out)
    print('\t\t-- the lift parks at floor f0 infinitely many times', file=out)
    print('\t\tf0 | '+sb_formula, file=out)
    print('', file=out)

    # result
    print('-- result: realizable', file=out)

    # close file
    out.close()






def generate_problem_slugs(index):
    filename = 'lift_controller_'+str(index)+'.structuredslugs'
    out = open(filename, 'w')
    
    # Preamble
    print('# Lift Controller with n='+str(index)+' floors.', file=out)

    # [INPUT]
    print('[INPUT]', file=out)
    print('\t# buttons', file=out)
    for i in range(0,index):
        print('\tb'+str(i), file=out)
    print('', file=out)
    
    # VAR
    print('[OUTPUT]', file=out)
    print('\t# floors', file=out)
    for i in range(0,index):
        print('\tf'+str(i), file=out)
    print('', file=out)

    # GR1SPEC
    print('# ENVIRONMENT', file=out)

    # [ENVINIT]
    print('[ENV_INIT]', file=out)
    print('\t# initially, on all floors there are no requests', file=out)
    print('\t', end='', file=out)
    for i in range(0,index):
        end_str = ' ' if ( i==index-1 ) else ' & ' 
        print('!b'+str(i)+end_str, end='', file=out)
    print('', file=out)

    # ENVTRANS
    print('[ENV_TRANS]', file=out)

    # once a request has been fulfilled, it is removed 
    print('\t# once a request has been fulfilled, it is removed', file=out)
    for i in range(0,index):
        print('\t(b'+str(i)+' & f'+str(i)+' -> X !b'+str(i)+')', file=out)
    # once the lift has been called on some floor, the request
    # cannot be withdrawn
    print('\t# once the lift has been called on some floor, the request', file=out);
    print('\t# cannot be withdrawn', file=out);
    for i in range(0,index):
        print('\t(b'+str(i)+' & !f'+str(i)+' -> X b'+str(i)+')', file=out)
    
    # ENV_LIVENESS
    print('[ENV_LIVENESS]', file=out)
    print('\tTRUE', file=out)
    print('', file=out)


    # CONTROLLER
    print('# CONTROLLER', file=out)

    # SYSINIT
    print('[SYS_INIT]', file=out)
    print('\t# initially the lift is on the first floor', file=out)
    print('\t f0 & ', end='', file=out)
    for i in range(1,index):
        end_str = ' ' if (i == index-1) else ' & '
        print('!f'+str(i)+end_str, end='', file=out)
    print('', file=out)

    # SYSTRANS
    print('[SYS_TRANS]', file=out)
    print('\t# the lift should *not* move *up* unless some button is pressed', file=out)
    up_formula = '('
    for i in range(0,index-1):
        end_str = ' ' if (i == index-2) else ' | '
        up_formula += '(f'+str(i)+' & X f'+str(i+1)+')'+end_str
    up_formula += ')'
    sb_formula = '('
    for i in range(0,index):
        end_str = ' ' if (i == index-1) else ' | ' 
        sb_formula += 'b'+str(i)+end_str
    sb_formula += ')'
    print('\t'+up_formula+' -> '+sb_formula, file=out)

    # mutual exclusion
    print('\t# mutual exclusion', file=out)
    for i in range(0, index):
        for j in range(0,i):
            print('\t!(f'+str(i)+' & f'+str(j)+')', file=out)

    # the lift can move at most one floor at a time (move up, move
    # down, or not move)
    print('\t# the lift can move at most one floor at a time (move up, move',file=out)
    print('\t# down, or not move)', file=out)
    for i in range(0,index):
        if i == 0:
            print('\t(f'+str(i)+' -> X(f'+str(i)+' | f'+str(i+1)+'))', file=out)
        elif i == index-1:
            print('\t(f'+str(i)+' -> X(f'+str(i-1)+' | f'+str(i)+'))', file=out)
        else:
            print('\t(f'+str(i)+' -> X(f'+str(i)+' | f'+str(i-1)+' | f'+str(i+1)+'))', file=out)

    # SYSFAIR
    for i in range(0,index):
        print('[SYS_LIVENESS]', file=out)
        print('\t# all the requests are eventually satisfied', file=out)
        print('\t(b'+str(i)+' -> f'+str(i)+')', file=out)

    # SYSFAIR
    # either some button is pressed infinitely many times, or
    # the lift parks at floor f0 infinitely many times
    print('[SYS_LIVENESS]', file=out)
    print('\t# either some button is pressed infinitely many times, or', file=out)
    print('\t# the lift parks at floor f0 infinitely many times', file=out)
    print('\tf0 | '+sb_formula, file=out)
    print('', file=out)

    # result
    print('# result: realizable', file=out)

    # close file
    out.close()






def main(argv):
    index = ''
    begin = ''
    end = ''
    grace = False
    slugs = False
    try:
        opts, args = getopt.getopt(argv,"hgsi:b:e:")
    except getopt.GetoptError:
        print('test.py -i <index> -b <begin> -e <end> [-g] [-s]')
        sys.exit(2)
    for opt, arg in opts:
        if opt == '-h':
            print('test.py -i <index> -b <begin> -e <end> [-g] [-s]')
            print('-i <index>\t: generates the lift_controller problem with '
                  'the specified index')
            print('-b\t\t: begin index')
            print('-e\t\t: end index')
            print('-g\t\t: dumps the problem in GRACE format')
            print('-s\t\t: dumps the problem in SLUGS format')
            sys.exit()
        elif opt in ("-i", "--index"):
            index = arg
        elif opt in ("-b", "--begin"):
            begin = arg
        elif opt in ("-e", "--end"):
            end = arg
        elif opt in ("-g", "--grace"):
            grace = True
        elif opt in ("-s", "--slugs"):
            slugs = True
        else:
            print('Option %s not valid', opt)
            sys.exit(1)
    # check on the options
    index_number = 0
    if(not index and not begin and not end):
        print('*** Please specify the index with -i <index>')
        sys.exit()
    if index:
        index_number = int(index)
        if(index_number < 2):
            print('*** The index must be >= 2')
            sys.exit()
    if(index and (begin or end)):
        print('*** Please specify exactly one between -i and <-b,-e>')
        sys.exit()
    if(not index and (not begin or not end)):
        print('*** Please specify either -i or *both* -b and -e')
        sys.exit()
    if(begin and int(begin)<2):
        print('*** The starting index must be >= 2')
        sys.exit()
    if(not grace and not slugs):
        print('*** Please specify at least one between -g and -s')
        sys.exit()

    
    # main
    if index:
        if grace:
            generate_problem_grace(index_number)
        if slugs:
            generate_problem_slugs(index_number)
    else:
        for i in range(int(begin),int(end)+1):
            if grace:
                generate_problem_grace(i)
            if slugs:
                generate_problem_slugs(i)




if __name__ == "__main__":
   main(sys.argv[1:])
