import os, sys
import subprocess
from subprocess import Popen, PIPE, STDOUT


BASE_DIR = os.path.dirname(os.path.dirname(os.path.realpath(__file__)))


def run_grace(filename):
    '''
    Runs the GRACE tool
    '''
    # execute nuXmv
    nuXmv_cmd = subprocess.run(
        ["docker", "run", 
         "--rm", "-it", "-v", BASE_DIR+":"+BASE_DIR, "-w", BASE_DIR, "grace:latest", 
         "-i", filename,
         "-u", "20"
         ],
        capture_output=True,
        text=True
    )
    nuXmv_result = nuXmv_cmd.stdout.splitlines()
    for line in nuXmv_result:
        if line.startswith("Specification found to be realizable"):
            return 'realizable'
        elif line.startswith("The upperbound 20 has been reached"):
            return 'unrealizable'
        else:
            continue



def get_outcome(filename):
    '''
    Gets the outcome from the file. The outcome is written in the file 
    in the following format:
        -- result: realizable 
        -- result: unrealizable 
    '''
    # read the file
    with open(filename, 'r') as filename_handle:
        file_content = filename_handle.read()
        if "-- result: realizable" in file_content:
            return 'realizable'
        if "-- result: unrealizable" in file_content:
            return 'unrealizable'
        else:
            print(filename+' does not contain the expected result')
            exit(1)



def main():
    # find test files in current dir
    find_cmd = subprocess.run(
        ["find", "test/GR-EBR", "-name", "*.smv"], 
        capture_output=True, # equiv. to stdout=PIPE e stderr=PIPE
        text=True
    )
    
    # splits new lines
    find_result = find_cmd.stdout.splitlines()
    # print header
    print("Test.name\tGRACE.result")
    # iterates over lines
    for filename in find_result:
        result = ''
        output_grace = run_grace(filename)
        output_expected = get_outcome(filename)
        if output_grace != 'realizable' and output_grace != 'unrealizable':
            result = 'ERROR'
        elif output_grace != output_expected:
            result = 'ERROR'
        else:
            result = 'ok'
        print(filename+'\t'+output_grace+'\t'+result)



if __name__ == '__main__':
    main()
